clc; close all; 
%clear all;
clearvars -except Kp Kd Uon Uoff Km Tm; 
%% Variables needed if live script is not used. Comment out otherwise

% Kp = 25; %P-Gain, default 25
% 
% Kd = 50; %D-Gain, default 25
% 
% %PWPF parameters
% Uon = 0.5;      %on-value, default 0.5
% Uoff = 0.3;     %off-value, default 0.3
% Km = 4;         %Filter gain, default 4
% Tm = 0.3;       %Filter time constant, default 0.3



%% Initialization

I = 5.58 *10^(-3); %[kg*m^2], inertia, default 6 * 10^(-3)
F = 20;         %[mN], thrust, default 20
Kpx = 1;
Kpy = 1;
Kpz = 1;

F_T = F;
Ixx = I;
%h = Uon-Uoff; %hysteresis width

%MinImpWidth =-Tm*log(1-h/Km)*1000; %Minimum impulse width

tmax = 35;
RefAngle = [0];
RefAngle2 = [pi/6];
RefAngleStr =('30@5s, 0@20s');
RefTime.time = [0; 5;5;20;20;400];
RefTime.signals.values = [RefAngle;RefAngle;RefAngle2;RefAngle2;RefAngle;RefAngle];

%Iyy = 6 *10^(-3); %kg*m^2
%Izz = 6 *10^(-3); %kg*m^2
%% Run simulation

sim('SatelliteADCS_Model');
t=[ans.tout];
%QuatDiff = [ans.QuatDiff];
ProducedTorques = [ans.ProducedTorques*1000];
%CommandedTorques = [ans.CommandedTorques];
Euler = [ans.Euler];
%Omega = [ans.Omega];
%Alpha = [ans.Alpha];

%% Post processing

ValveOpenTime = 0;
ThrusterFirA = 0;
ThrusterFirB = 0;
t_90 = NaN;
t_s = NaN;
ind_t_90 = 1;
ind_t_s = 1;
Overshoot = NaN;

for i=2:size(Euler,1) % Calcuate thruster activity
    if ProducedTorques(i,1)> 0 && ProducedTorques(i-1,1)== 0
        ThrusterFirA = ThrusterFirA +1;
    elseif ProducedTorques(i,1)< 0 && ProducedTorques(i-1,1)== 0
        ThrusterFirB = ThrusterFirB +1;
    end
    
    if ProducedTorques(i,1)> 0 && ProducedTorques(i-1,1)> 0
        ValveOpenTime = ValveOpenTime + t(i,1)-t(i-1,1);
    elseif ProducedTorques(i,1)< 0 && ProducedTorques(i-1,1)< 0
        ValveOpenTime = ValveOpenTime + t(i,1)-t(i-1,1);
    end
        
end

for i=2:size(Euler,1) % Calculate rise time t_90
    if (Euler(i,1)>0.9* RefAngle2(1))
        ind_t_90 = i;
        t_90 = round(t(ind_t_90)-5,1);
        break
    end
end


ind_20 = interp1(t,1:length(t),20,'nearest')-1;% Calculate index of 20sec (to ignore it when computing the settling time)
ind_5 = interp1(t,1:length(t),5,'nearest')+1;

for i=ind_5:ind_20 
    if (min(Euler(i:ind_20,1))>0.95* RefAngle2(1)) && (max(Euler(i:ind_20,1))<1.05* RefAngle2(1))
        ind_t_s = i;
        t_s = round(t(ind_t_s)-5,1);
        break
    end
end

 

if max(Euler(:,1)) > RefAngle2(1)
    Overshoot = round(100*(max(Euler(:,1))-RefAngle2(1))/RefAngle2(1),1);
end

%% Plotting
set(0 , 'Units' , 'inches')
size_screen = get(0 , 'ScreenSize');
set(0 , 'Units' , 'pixels')
figure('Name', 'Torque');

hold on
plot(t,ProducedTorques(:,1),'LineWidth',1.5)
set(gca,'FontSize',20)
set(gcf, 'Position', get(0, 'Screensize'));
set(gcf,'Visible','on')

xlabel('Time [s]')
ylabel ('Torque [mNm]')

title('Produced Torque')

grid on;
hold off;


figure('Name', 'Angle');
hold on

plot(RefTime.time,180/pi*RefTime.signals.values(:,1),'LineWidth',1.5);

plot(t,180/pi*Euler(:,1),'LineWidth',1.5)
if ind_t_90 > 1
    plot(t(ind_t_90), 180/pi*Euler(ind_t_90,1),'.', 'MarkerSize',30);
    text(t(ind_t_90)+0.25, 180/pi*Euler(ind_t_90,1)-1,'t_{90}')
end
if ind_t_s > 1
    plot(t(ind_t_s), 180/pi*Euler(ind_t_s,1),'.', 'MarkerSize',30);
    text(t(ind_t_s), 180/pi*Euler(ind_t_s,1)-1.5,' t_{s}')
end
xlim([0 35]);
ylim([-10 40]);
xlabel('Time [s]')
ylabel ('Angle []')
title('Angular Position');
legend('Reference Signal', 'Response','Location', 'northwest')
set(gca,'FontSize',20)
set(gcf, 'Position', get(0, 'Screensize'));
set(gcf,'Visible','on')
  
str1 = {'\bullet PD',['K_p = ', num2str(Kp),'  |  K_d = ', num2str(Kd)] ,'','','\bullet PWPF', ['K_m = ', num2str(Km)],['U_{on} = ', num2str(Uon),'  |  U_{off} = ', num2str(Uoff)]};
str2 = {'\bullet Valves Usage', ['Time = ', num2str(round(ValveOpenTime,2)),'s'], ['Openings = ', num2str(ThrusterFirA),' | ', num2str(ThrusterFirB)],'','\bullet Performance', ['t_{90} = ', num2str(t_90),'s | t_{s} = ', num2str(t_s),'s'],['Overshoot = ', num2str(Overshoot),'%']};

dim1 = [.65 .69 .12 .22];
dim2 = [.7775 .69 .12 .22];
dim3 = [.9 .9 .1 .1];
a= annotation('textbox',dim1,'String',str1,'FitBoxToText','off');
a.FontSize= 12 * size_screen(3)/17.45;
b= annotation('textbox',dim2,'String',str2,'FitBoxToText','off');
b.FontSize= 12 * size_screen(3)/17.45;
% c= annotation('text',dim3,'String',{date},'FitBoxToText','off');
% c.FontSize= 12 * size_screen(3)/17.45;

c = text(33,-9,date);
c.FontSize= 7 * size_screen(3)/17.45;


grid on;
hold off;

%% Playground
%plot(t,180/pi*Euler(:,2),'LineWidth',2)
%plot(t,180/pi*Euler(:,3),'LineWidth',2)
%yline(180/pi*RefAngle(1,1),'-.b','LineWidth',2);
%yline(180/pi*RefAngle(1,2),'-.k','LineWidth',2);
%yline(180/pi*RefAngle(1,3),'-.r','LineWidth',2);